from collections.abc import Awaitable, Callable
from dataclasses import asdict, dataclass
from json import JSONDecoder, JSONEncoder
from pathlib import Path
from typing import Any, Dict, Generator, List, cast

import asyncio
from magic import Magic
from redis.asyncio import Redis
from redis.asyncio.connection import ConnectionPool


class RedisManager:
    def __init__(self, connection_url: str):
        self.connection_pool = ConnectionPool.from_url(connection_url)

    async def __aenter__(self):
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()
    
    async def close(self):
        await cast(Awaitable, self.connection_pool.disconnect())

    def get_client(self):
        return Redis(connection_pool=self.connection_pool)

@dataclass
class MediaDTO:
    inode: int
    paths: list[Path]
    is_transcoded: bool = False

    def open(self):
        return open(self.paths[0], "rb")
    
    def size(self) -> int:
        return self.paths[0].stat().st_size

class MediaDAO:
    def __init__(self, redis_manager: RedisManager):
        self.redis_client = redis_manager.get_client()

    async def get_media_by_inode(self, inode: int) -> MediaDTO:
        result = await cast(Awaitable[MediaDTO], self.redis_client.json(encoder=json_encoder, decoder=json_decoder).get(f"media_info:{inode}"))
        if not result:
            raise ValueError(f"Media with inode {inode} not found")
        return result

    async def get_all_inodes(self) -> list[int]:
        keys = await self.redis_client.keys("media_info:*")
        return [int(key.decode().split(":")[2]) for key in keys]
    
    async def set_media(self, media: MediaDTO) -> None:
        await cast(Awaitable[int], self.redis_client.json(encoder=json_encoder, decoder=json_decoder).set(f"media_info:{media.inode}", "$", asdict(media)))

    async def batch_set_media(self, media_list: list[MediaDTO]) -> None:
        async with self.redis_client.pipeline() as pipe:
            for media in media_list:
                await cast(Awaitable[int], pipe.json(encoder=json_encoder, decoder=json_decoder).set(f"media_info:{media.inode}", "$", asdict(media)))
            await pipe.execute()

    async def delete_media(self, media: MediaDTO) -> None:
        await cast(Awaitable[int], self.redis_client.delete(f"media_info:{media.inode}"))

    async def mark_as_transcoded(self, media: MediaDTO) -> None:
        await cast(Awaitable[int], self.redis_client.json(encoder=json_encoder, decoder=json_decoder).set(f"media_info:{media.inode}", "$.is_transcoded", True))
    
    async def is_transcoded(self, inode: int) -> bool:
        return await cast(Awaitable[List[bool]], self.redis_client.json(encoder=json_encoder, decoder=json_decoder).get(f"media_info:{inode}", "$.is_transcoded")) == [True]

class JSONEncoderImpl(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Path):
            return obj.as_uri()
        return super().default(obj)

def object_hook(dict: Dict[str, Any]):
    return MediaDTO(
        paths=[Path.from_uri(v) for v in dict["paths"]],
        inode=int(dict["inode"]),
        is_transcoded=dict["is_transcoded"],
    )

json_encoder = JSONEncoderImpl()
json_decoder = JSONDecoder(object_hook=object_hook)
    

def is_media_file(path: Path) -> bool:
    return mime_detector.from_file(path).startswith("video/")

mime_detector = Magic(mime=True)
