from argparse import ArgumentParser
from ast import Dict
from pathlib import Path
from typing import Generator

import asyncio

from auto_transcoder import tasks
from auto_transcoder.model import MediaDAO, MediaDTO, RedisManager
from auto_transcoder.web import run as run_web_app

def walk_directory(path: Path, bin: Path | None) -> Generator[Path, None, None]:
    for root, _, files in path.walk():
        if bin and root.is_relative_to(bin):
            continue
        for file_name in files:
            yield Path(root, file_name)

async def main(directory_to_watch: Path, redis_connection_url: str, recycle_bin: Path | None = None):
    print(f"Watching {directory_to_watch} for media files.")
    print(f"Original files will be moved to recycle bin: {recycle_bin}.")

    if not directory_to_watch.exists():
        print(f"Directory {directory_to_watch} does not exist")
        exit(1)

    media_files_by_inode: dict[int, list[Path]] = {}
    for path in walk_directory(directory_to_watch, recycle_bin):
        inode = path.stat().st_ino
        if inode not in media_files_by_inode:
            media_files_by_inode[inode] = []
        media_files_by_inode[inode].append(path)

    async with RedisManager(redis_connection_url) as redis_manager:
        media_dao = MediaDAO(redis_manager)
        media_files_by_inode = {k: v for k, v in media_files_by_inode.items() if not await media_dao.is_transcoded(k)}
        await media_dao.batch_set_media([MediaDTO(inode=i, paths=p, is_transcoded=False) for i, p in media_files_by_inode.items()])
        for inode in media_files_by_inode.keys():
            print(f"Sent transcode task for inode {inode}")
            tasks.transcode_media_task.delay(inode, recycle_bin.as_uri() if recycle_bin else None)
    
        run_web_app(redis_manager=redis_manager, host='0.0.0.0', port=5000, use_reloader=False)

if __name__ == '__main__':
    parser = ArgumentParser(description='Auto Transcoder Server')
    
    parser.add_argument('directory_to_watch', type=str, help='Directory to watch for media files')
    parser.add_argument('--recycle-bin', type=str, default=None, help='Recycle bin directory for original files')
    
    args = parser.parse_args()

    recycle_bin = Path(args.recycle_bin) if args.recycle_bin else None

    asyncio.run(main(Path(args.directory_to_watch), args.redis_connection_url, recycle_bin))
